import { validate } from "./validation";
import { Control, VoicemeeterType } from "../VMR/Control";
import { throwError } from "../utils/Error";

export function minType(minType: VoicemeeterType) {
  return (object: any, name: string, descriptor: PropertyDescriptor) => {
    const keys = ["set", "get", "value"] as (keyof typeof descriptor)[];
    for (const key of keys) {
      if (descriptor[key] === undefined) continue;
      const originalFunction = descriptor[key];
      descriptor[key] = function (this: Control, ...args: any[]) {
        validate.bind(this)();
        if (this.voicemeeterType < minType)
          return throwError(
            `Used property/method with name: ${name} is not supported by this type of Voicemeeter. Your type: ${
              VoicemeeterType[this.voicemeeterType]
            } Minimal required type is ${VoicemeeterType[minType]}`
          );
        return originalFunction.apply(this, args);
      };
    }
    return descriptor;
  };
}
