import { Control, VoicemeeterType } from "./VMR/Control";
import { logger, Logger } from "./Logger";
import { VOICEMEETER_REMOTE_DLL } from "./variables";
import { throwError } from "./utils/Error";

if (VOICEMEETER_REMOTE_DLL === undefined)
  throwError("The path for VM Remote dll is not specified");
let control: Control = new Control(VOICEMEETER_REMOTE_DLL!);
let interval: NodeJS.Timeout;

function exit(eventName: (typeof exitEvents)[0]) {
  clearInterval(interval);
  if (control !== undefined && control.loggedIn) control.logout();
  logger.log({ tag: `Signal: ${eventName.name}` }, "Logout");
  process.exit();
}

const exitEvents = [
  { name: "SIGINT" },
  { name: "SIGABRT" },
  { name: "SIGKILL" },
  { name: "SIGUSR1" },
  { name: "SIGUSR2" },
  { name: "beforeExit" },
  { name: "uncaughtException" },
  { name: "unhandledRejection" },
] as { name: NodeJS.Signals }[];

for (const event of exitEvents) {
  process.on(event.name, () => exit(event));
}

logger.log({}, control.availableInputDevices);
