import { promises as fs } from "fs";

interface Parameter {
  name: string;
  type: string;
}

interface ParsedFunction {
  name: string;
  params: Parameter[];
  returnType: string;
  rawString: string;
}

export class Parser {
  constructor(private filePath: string) {}
  public async parse() {
    const data = await fs.readFile(this.filePath, { encoding: "utf-8" });
    const lines = data.split("\n").map((l) => l.trim());
    let i = 0;
    let isComment = false;
    const functionArray = [] as ParsedFunction[];
    while (++i < lines.length) {
      const line = lines[i];
      if (!isComment && line.includes("/*")) {
        isComment = true;
        continue;
      }
      if (isComment && line.includes("*/")) {
        isComment = false;
        continue;
      }
      if (line.includes("typedef") || line.includes("//") || line.includes("#"))
        continue;
      if (line.includes(" __stdcall ") && line.endsWith(");")) {
        const split = line
          .substring(0, line.length - ");".length)
          .split(" __stdcall ");

        const returnType = split[0];
        const [functionName, parametersString] = split[1].split("(");
        let params = parametersString
          .split(", ")
          .map((p) => p.split(/\s(?!\*)/));
        functionArray.push({
          rawString: line,
          name: functionName,
          params: params.map((p) => ({ name: p[1], type: p[0] })),
          returnType,
        });
      }
    }
    return functionArray;
  }
}

